package lib_allinpay

type ConsumeApply struct {
	PayerId             string     `json:"payerId"`
	BizOrderNo          string     `json:"bizOrderNo"`
	RecieverId          string     `json:"recieverId"`
	Amount              int64      `json:"amount"`
	Fee                 int64      `json:"fee"`
	ValidateType        uint8      `json:"validateType"`
	BackUrl             string     `json:"backUrl"`
	OrderExpireDatetime string     `json:"orderExpireDatetime"`
	PayMethod           *PayMethod `json:"payMethod"`
	IndustryCode        string     `json:"industryCode"`
	IndustryName        string     `json:"industryName"`
	Source              int        `json:"source"`
	Summary             string     `json:"summary"`
}

type PayMethod struct {
	WECHATPAY_MINIPROGRAM_ORG *WECHATPAY_MINIPROGRAM_ORG `json:"WECHATPAY_MINIPROGRAM_ORG"`
}

type WECHATPAY_MINIPROGRAM_ORG struct {
	VspCusid string `json:"vspCusid"`
	SubAppid string `json:"subAppid"`
	LimitPay string `json:"limitPay"`
	Amount   int64  `json:"amount"`
	Acct     string `json:"acct"`
}

func (ConsumeApply) apiName() string {
	return "allinpay.yunst.orderService.consumeApply"
}

type WxMiniPayOpenResponse struct {
	OpenResponse
	Data WxMiniPayOpenResponseData
}

type WxMiniPayOpenResponseData struct {
	PayInterfaceOutTradeNo string
	OrderNo                string
	PayInfo                string
	BizUserId              string
	BizOrderNo             string
}

type GetOrderDetail struct {
	BizOrderNo string `json:"bizOrderNo"`
}

func (GetOrderDetail) apiName() string {
	return "allinpay.yunst.orderService.getOrderDetail"
}

type GetOrderDetailOpenResponse struct {
	OpenResponse
	Data GetOrderDetailOpenResponseData
}

type GetOrderDetailOpenResponseData struct {
	Amount         string
	BizOrderNo     string
	BuyerBizUserId string
	OrderStatus    string
	OrderNo        string
	ErrorMessage   string
	PayDatetime    string
}

type Refund struct {
	BizUserId     string          `json:"bizUserId"`
	BizOrderNo    string          `json:"bizOrderNo"`
	OriBizOrderNo string          `json:"oriBizOrderNo"`
	Amount        int64           `json:"amount"`
	FeeAmount     int64           `json:"feeAmount"`
	RefundType    string          `json:"refundType"`
	BackUrl       string          `json:"backUrl"`
	RefundList    []RefundAccount `json:"refundList"`
}

type RefundAccount struct {
	AccountSetNo string `json:"accountSetNo"`
	BizUserId    string `json:"bizUserId"`
	Amount       int64  `json:"amount"`
}

func (Refund) apiName() string {
	return "allinpay.yunst.orderService.refund"
}

type QueryReserveFundBalance struct {
}

func (QueryReserveFundBalance) apiName() string {
	return "allinpay.yunst.merchantService.queryReserveFundBalance"
}

type QueryReserveFundBalanceResponse struct {
	OpenResponse
	Data QueryReserveFundBalanceResponseData
}

type QueryReserveFundBalanceResponseData struct {
	AccountName string
	AccountNo   string
	Balance     string
	DefClr      string
}

type QueryBalance struct {
	AccountSetNo string `json:"accountSetNo"`
	BizUserId    string `json:"bizUserId"`
}

func (QueryBalance) apiName() string {
	return "allinpay.yunst.orderService.queryBalance"
}

type QueryBalanceResponse struct {
	OpenResponse
	Data QueryBalanceResponseData
}

type QueryBalanceResponseData struct {
	AllAmount     string
	FreezenAmount string
}

type WithdrawApply struct {
	AccountSetNo        string `json:"accountSetNo"`
	BizUserId           string `json:"bizUserId"`
	BizOrderNo          string `json:"bizOrderNo"`
	Amount              int64  `json:"amount"`
	Fee                 int64  `json:"fee"`
	ValidateType        uint8  `json:"validateType"`
	BackUrl             string `json:"backUrl"`
	OrderExpireDatetime string `json:"orderExpireDatetime"`
	BankCardNo          string `json:"bankCardNo"`
	BankCardPro         uint8  `json:"bankCardPro"`
	WithdrawType        string `json:"withdrawType"`
	IndustryCode        string `json:"industryCode"`
	IndustryName        string `json:"industryName"`
	Source              uint8  `json:"source"`
	Summary             string `json:"summary"`
}

func (WithdrawApply) apiName() string {
	return "allinpay.yunst.orderService.withdrawApply"
}

type WithdrawApplyResponseData struct {
	PayInterfaceOutTradeNo string
	BuyerBizUserId         string
	Amount                 int64
	OrderNo                string
	ExtendInfo             string
	PayDatetime            string
	Acct                   string
	BizOrderNo             string
	Status                 string
}

type QueryInExpDetail struct {
	BizUserId     string `json:"bizUserId"`
	BizOrderNo    string `json:"bizOrderNo"`
	DateStart     string `json:"dateStart"`
	DateEnd       string `json:"dateEnd"`
	StartPosition int    `json:"startPosition"`
	QueryNum      int    `json:"queryNum"`
	TradeType     string `json:"tradeType"`
}

func (QueryInExpDetail) apiName() string {
	return "allinpay.yunst.orderService.queryInExpDetail"
}

type QueryInExpDetailResponse struct {
	OpenResponse
	Data QueryInExpDetailResponseData
}

type QueryInExpDetailResponseData struct {
	BizUserId   string
	TotalNum    string
	InExpDetail []InExpDetail
	ExtendInfo  string
}

type InExpDetail struct {
	TradeNo          string
	AccountSetName   string
	ChangeTime       string
	CurAmount        int64
	OriAmount        int64
	ChgAmount        int64
	CurFreezenAmount int64
	BizOrderNo       string
	TradeType        string
	Type             string
	Remark           string
	ExtendInfo       string
}

type GetPayeeFundsInTransitDetail struct {
	BizUserId    string `json:"bizUserId"`
	BizOrderNo   string `json:"bizOrderNo"`
	DateStart    string `json:"dateStart"`
	DateEnd      string `json:"dateEnd"`
	AccountSetNo string `json:"accountSetNo"`
}

func (GetPayeeFundsInTransitDetail) apiName() string {
	return "allinpay.yunst.orderService.getPayeeFundsInTransitDetail"
}

type PayNotifyData struct {
	Amount                 *int    `json:"amount"`
	OrderNo                *string `json:"orderNo"`
	ChannelFee             *string `json:"channelFee"`
	ChannelPaytime         *string `json:"channelPaytime"`
	ExtendInfo             *string `json:"extendInfo"`
	PayInterfaceOutTradeNo *string `json:"payInterfaceOutTradeNo"`
	BuyerBizUserId         *string `json:"buyerBizUserId"`
	Cusid                  *string `json:"cusid"`
	PayInterfacetrxcode    *string `json:"payInterfacetrxcode"`
	PayDatetime            *string `json:"payDatetime"`
	Acct                   *string `json:"acct"`
	BizOrderNo             *string `json:"bizOrderNo"`
	Status                 *string `json:"status"`
	RefundWhereabouts      *int    `json:"refundWhereabouts"`
	OriOrderNo             *string `json:"oriOrderNo"`
	OriBizOrderNo          *string `json:"oriBizOrderNo"`
}
