package util

import (
	"encoding/json"
	"git.168cad.top/zhengqiuyun/rym-util/exception"
	"github.com/gin-gonic/gin"
	"github.com/shopspring/decimal"
	"math/rand"
	"strconv"
)

// 生成: 时间戳 + 设置前缀 ＋ 随即字符串
const alphanum = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"

func FandomString(strlen int) string {
	result := make([]byte, strlen)
	for i := 0; i < strlen; i++ {
		result[i] = alphanum[rand.Intn(len(alphanum))]
	}
	return string(result)
}

func ParamUint32(c *gin.Context, key string) uint32 {
	v := c.Query(key)
	if v == "" {
		return 0
	}
	i, err := strconv.Atoi(v)
	if err != nil {
		exception.ThrowsErrS("参数类型错误：请传入正确的数字")
	}
	return uint32(i)
}

func ParamUint8(c *gin.Context, key string) uint8 {
	v := c.Query(key)
	if v == "" {
		return 0
	}
	i, err := strconv.Atoi(v)
	if err != nil {
		exception.ThrowsErrS("参数类型错误：请传入正确的数字")
	}
	return uint8(i)
}

func Param(c *gin.Context, key string) string {
	v := c.Query(key)
	return v
}

func Path(c *gin.Context, key string) string {
	v := c.Param(key)
	if v == "" {
		exception.ThrowsErrS("参数[" + key + "]必填")
	}
	return v
}

// Strval 获取变量的字符串值
// 浮点型 3.0将会转换成字符串3, "3"
// 非数值或字符类型的变量将会被转换成JSON格式字符串
func Strval(value interface{}) string {
	// interface 转 string
	var key string
	if value == nil {
		return key
	}

	switch value.(type) {
	case float64:
		ft := value.(float64)
		key = strconv.FormatFloat(ft, 'a', -1, 64)
	case float32:
		ft := value.(float32)
		key = strconv.FormatFloat(float64(ft), 'a', -1, 64)
	case int:
		it := value.(int)
		key = strconv.Itoa(it)
	case uint:
		it := value.(uint)
		key = strconv.Itoa(int(it))
	case int8:
		it := value.(int8)
		key = strconv.Itoa(int(it))
	case uint8:
		it := value.(uint8)
		key = strconv.Itoa(int(it))
	case int16:
		it := value.(int16)
		key = strconv.Itoa(int(it))
	case uint16:
		it := value.(uint16)
		key = strconv.Itoa(int(it))
	case int32:
		it := value.(int32)
		key = strconv.Itoa(int(it))
	case uint32:
		it := value.(uint32)
		key = strconv.Itoa(int(it))
	case int64:
		it := value.(int64)
		key = strconv.FormatInt(it, 10)
	case uint64:
		it := value.(uint64)
		key = strconv.FormatUint(it, 10)
	case string:
		key = value.(string)
	case []byte:
		key = string(value.([]byte))
	default:
		newValue, _ := json.Marshal(value)
		key = string(newValue)
	}

	return key
}

func IsEmpty(s *string) bool {
	return s == nil || *s == ""
}

func IsNil(i interface{}) bool {
	if i == nil {
		return true
	}
	return false
}

func YuanToFen(money float64) int64 {
	v12 := decimal.NewFromFloat(float64(money) * 100).Round(0).BigInt().Int64()
	return v12
}

func FenToYuan(money int64) float64 {
	v12, _ := decimal.NewFromFloat(float64(money) / 100).Round(2).Float64()
	return v12
}
