package util

import (
	"crypto/aes"
	"crypto/sha1"
	"encoding/hex"
	"errors"
	"strings"
)

//AesEncryptECBAllin 通联数据加密
func AesEncryptECBAllin(src, key string) (string, error) {
	data, err := AesEncryptECB(src, key)
	return strings.ToUpper(data), err
}

//AesDecryptECBAllin 通联数据解密
func AesDecryptECBAllin(src, key string) (string, error) {
	src = strings.ToLower(src)
	data, err := AesDecryptECB(src, key)
	return data, err
}

func AesEncryptECB(src, key string) (string, error) {
	data, err := aesEncryptECB([]byte(src), []byte(key))
	eStr := hex.EncodeToString(data)
	return eStr, err
}

func AesDecryptECB(src, key string) (string, error) {
	data, err := hex.DecodeString(src)
	if err != nil {
		return "", err
	}
	re, err := aesDecryptECB([]byte(data), []byte(key))
	return string(re), err
}

func aesEncryptECB(src []byte, key []byte) ([]byte, error) {
	key, err := aesSha1prng(key, 128) // 比示例一多出这一步
	if err != nil {
		return nil, err
	}

	cipher, _ := aes.NewCipher(generateKey(key))
	length := (len(src) + aes.BlockSize) / aes.BlockSize
	plain := make([]byte, length*aes.BlockSize)
	copy(plain, src)
	pad := byte(len(plain) - len(src))
	for i := len(src); i < len(plain); i++ {
		plain[i] = pad
	}
	encrypted := make([]byte, len(plain))
	// 分组分块加密
	for bs, be := 0, cipher.BlockSize(); bs <= len(src); bs, be = bs+cipher.BlockSize(), be+cipher.BlockSize() {
		cipher.Encrypt(encrypted[bs:be], plain[bs:be])
	}

	return encrypted, nil
}

func aesDecryptECB(encrypted []byte, key []byte) ([]byte, error) {
	key, err := aesSha1prng(key, 128) // 比示例一多出这一步
	if err != nil {
		return nil, err
	}

	cipher, _ := aes.NewCipher(generateKey(key))
	decrypted := make([]byte, len(encrypted))
	for bs, be := 0, cipher.BlockSize(); bs < len(encrypted); bs, be = bs+cipher.BlockSize(), be+cipher.BlockSize() {
		cipher.Decrypt(decrypted[bs:be], encrypted[bs:be])
	}

	trim := 0
	if len(decrypted) > 0 {
		trim = len(decrypted) - int(decrypted[len(decrypted)-1])
	}

	return decrypted[:trim], nil
}

func aesSha1prng(keyBytes []byte, encryptLength int) ([]byte, error) {
	a := sha1Key(sha1Key(keyBytes))
	maxLen := len(a)
	realLen := encryptLength / 8
	if realLen > maxLen {
		return nil, errors.New("密钥长度非法")
	}

	return a[0:realLen], nil
}

func sha1Key(data []byte) []byte {
	h := sha1.New()
	h.Write(data)
	return h.Sum(nil)
}

func generateKey(key []byte) (genKey []byte) {
	genKey = make([]byte, 16)
	copy(genKey, key)
	for i := 16; i < len(key); {
		for j := 0; j < 16 && i < len(key); j, i = j+1, i+1 {
			genKey[j] ^= key[i]
		}
	}
	return genKey
}
